
/**
 * An array-based, linked list of Objects.  
 *
 * @author Zach Tomaszewski
 * @date 06 Oct 2001
 */
public class ListArrayBasedLinks implements ListInterface {
 static final int DEFAULT_SIZE = 50;
 ArrayNode[] arrayFrame;
 int head, free, size;


 /**
  * Creates a new of default capacity.
  */
 public ListArrayBasedLinks() {
   this(DEFAULT_SIZE);
 }

 /**
  * Creates a new list of the specified capacity.
  */
 public ListArrayBasedLinks(int capacity) {
   this.arrayFrame = new ArrayNode[capacity];
   //fill with free nodes, except first and last indices
   int i; //need this below, outside of the for loop
   for (i = 0; i < capacity-1; i++){
     arrayFrame[i] = new ArrayNode(null, i+1);
   }
   //the last node with a dead end
   arrayFrame[i] = new ArrayNode(null);
   this.free = 0;
   this.head = -1;
   this.size = 0;
 }


 /**
  * Determines whether this list contains any elements.
  *
  * @returns true if there are no elements; false otherwise
  */
 public boolean isEmpty(){
   return (size==0);
 }


 /**
  * Returns the number of elements in this list.
  *
  * @returns The number of elements in this list.
  */
 public int size(){
   return size;
 }

 
 /**
  * Inserts an item at the specified position on the list, shifting all other
  * items in the list up one position.
  *
  * @param position  The position at which to insert the item
  * @param item      The Object to insert
  * @throws ListIndexOutOfBoundsException
  *             if (position < 1 || position > size() + 1)
  */
 public void add(int position, Object item)
                     throws ListIndexOutOfBoundsException, ListException{

   if (this.size >= arrayFrame.length) {
      //this list has reached capacity;
      throw new ListException("Cannot add item: " +
                                          "No more room in the list.");
   }else if (position < 1 || position > size() + 1){
      throw new ListIndexOutOfBoundsException("Cannot add time: " +
                                            "Position out of bounds.");
   }else if (position == 1) { //adding to head
      int newNodeIndex = free;
      free = arrayFrame[free].getNext();
      if (head == -1) { //new head
        arrayFrame[newNodeIndex] = new ArrayNode(item);
      }else{ //already a head; move it over
        arrayFrame[newNodeIndex] = new ArrayNode(item, head);
      }
      head = newNodeIndex;
      size++;
   }else{
      int prev = -1;
      int currIndex = head;
      for (int i = 1; i < position; i++){
          prev = currIndex;
          currIndex = arrayFrame[currIndex].getNext();
      }            
      int newNodeIndex = free;
      free = arrayFrame[free].getNext();
      arrayFrame[newNodeIndex] = new ArrayNode(item, currIndex);
      arrayFrame[prev].setNext(newNodeIndex);
      size++;
   }
 }


 /**
  * Returns the item at the specified position on the list.
  * The list is unchanged.
  *
  * @param position  The position to retrieve
  * @throws ListIndexOutOfBoundsException
  *             if (position < 1 || position > size())
  */
 public Object get(int position)
                     throws ListIndexOutOfBoundsException{
   if (position < 1 || position > size()){
      throw new ListIndexOutOfBoundsException("Cannot add time: " +
                                            "Position out of bounds.");
   }else{
      int tempIndex = head;
      for (int i = 1; i < position; i++){
        tempIndex = arrayFrame[tempIndex].getNext();
      }
      return arrayFrame[tempIndex].getItem();
   }
 }


 /**
  * Removes the item at the specified position from the list.
  * All items with higher positions are shifted down.
  *
  * @param position  The position to remove
  * @throws ListIndexOutOfBoundsException
  *             if (position < 1 || position > size() + 1)
  */
 public void remove(int position)
                      throws ListIndexOutOfBoundsException{
   if (position < 1 || position > size() + 1){
      throw new ListIndexOutOfBoundsException("Cannot add time: " +
                                            "Position out of bounds.");
   }else{
      int prev = -1;
      int currIndex = head;
      for (int i = 1; i < position; i++){
          prev = currIndex;
          currIndex = arrayFrame[currIndex].getNext();
      }

      if (prev < 0) { //removing at head
        head = arrayFrame[currIndex].getNext();
      }else {
        arrayFrame[prev].setNext(arrayFrame[currIndex].getNext());
      }
      arrayFrame[currIndex] = new ArrayNode(null, free);
      free = currIndex;
      size--;
   }
 }


 /**
  * Removes all items with this list and sets the size to zero.
  */
 public void removeAll(){
   //the quick way to do it; maintains the same list length though.
//   this(arrayFrame.length);
 }

}//end class

