/**
 * TestList:  A program for testing implementations of ListInterface
 * @author Dan Suthers & Xiaohua Chen
 * @version 1.0
 */

import java.io.*;
import java.util.*;

public class TestList {

    /**
     * Loops to prompt users for actions on the list using String
     * objects.  See menu. The entire list is printed each loop.
     * @param args Command line arguments are ignored. 
     * @return No return value.
     * @exception Throws IOException if an IO error occurs. 

     */

    private static void printMenu() {
        System.out.println("a <position> <token> - add string <token> at <position>");
        System.out.println("g <position> - get the string at the position indicated");
        System.out.println("r <position> - delete the string at <position>");
        System.out.println("s - get size of this vector or report empty if size = 0");
        System.out.println("e, x, empty line - exit");
    }

    public static void main(String[] args) throws IOException {

        ListArrayBasedLinks shoppingList = new ListArrayBasedLinks();

        // Where to read from, and utility variables for reading values 
        BufferedReader stdin = new BufferedReader(new InputStreamReader(System.in));
        String nextLine; 
        int pos = -1; // for compiler's sake
        String nextToken; 

        // Print introductory information.

        System.out.println("**** List Interface Tester ****");
        System.out.println("All <position> arguments are integers; <token> can be any token, for example:");
        System.out.println("command: a 1 poi");
        System.out.println("command: a 1 papayas");
        System.out.println("command: a 3 bananas");
        System.out.println("command: r 2");
        System.out.println("-------------------------------");
        printMenu();

        // Loop to print time and read in commands
        boolean exitRequested = false; // for loop exit
        do {

            System.out.print("command: "); 
            nextLine = stdin.readLine();
            try {
                if (nextLine.length() > 0) {
                    StringTokenizer input = new StringTokenizer(nextLine, " ,");
                    char command = (input.nextToken()).charAt(0); 
                    switch (command) {
                    case 'a':
                        pos = Integer.parseInt(input.nextToken());
                        nextToken = input.nextToken(); 
                        shoppingList.add(pos, nextToken);
                        break;
                    case 'g':
                        pos = Integer.parseInt(input.nextToken()); 
                        nextToken = (String)shoppingList.get(pos);
                        System.out.println(pos  + " has " + nextToken); 
                        break;
                    case 'r':
                        pos = Integer.parseInt(input.nextToken());
                        shoppingList.remove(pos); 
                        break;
                     case 's':
                        if (shoppingList.isEmpty()) {
                            System.out.println("The list is empty");
                        }
                        else {
                            System.out.println(shoppingList.size() + " things are on the list." );
                        }
                        break;
                    case 'e': case 'x': 
                        exitRequested = true; 
                        break; 
                    default: 
                        System.out.println("Command not recognized. Choose one of the letters:"); 
                        printMenu(); 
                    }
                }
            }
            catch (NumberFormatException e) {
                System.out.println("There was an error parsing an integer; please try again"); 
            }
            catch (NoSuchElementException e) {
                System.out.println("You need to enter an integer after that command."); 
            }
            catch (ListIndexOutOfBoundsException e) {
                System.out.println("Position " + pos + " is invalid (out of range)"); 
            }
            printTheList(shoppingList);
        } while (!exitRequested && (nextLine.length() > 0)); 
    }
    
    private static void printTheList(ListInterface theList) {
        // either say it's empty or iterate to print each position
        int size = theList.size();
        if (size == 0) {
            System.out.println("The list is empty."); 
        }
        else {
            for (int i = 1; i <= size; i++) { 
                System.out.println((i) + ": " + theList.get(i));
            }
        }
    }
}
